package com.emato.eoss.msg.resp;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @author Scott Chen
 * @date 2017/4/20
 */
public class BizData implements Serializable {

    private static final long serialVersionUID = -2649758273960153111L;

    // 当前第几页
    private int page;
    // 总页数
    private int totalPage;
    // 单页记录数
    private int count;
    // 总记录数
    private int totalCount;
    // 查询数据
    private Object rows;

    private BizData() {
    }

    public int getPage() {
        return page;
    }

    public int getTotalPage() {
        return totalPage;
    }

    public int getCount(){
        return count;
    }

    public int getTotalCount() {
        return totalCount;
    }

    public Object getRows() {
        return rows;
    }

    public static Builder builder() {
        return new Builder();
    }
    public static Builder builder(List rows) {
        return new Builder(rows);
    }
    public static Builder builder(int page, int totalPage, int count, int totalCount, List rows) {
        return new Builder(page, totalPage, count, totalCount, rows);
    }
    public static <T> Builder builder(T t, Class<T> type) {
        return new Builder(t, type);
    }
    public static <T> Builder builder(int page, int totalPage, int count, int totalCount, T t, Class<T> type) {
        return new Builder(page, totalPage, count, totalCount, t, type);
    }

    /**
     * 消息构建器类
     */
    public static class Builder {
        private int page;
        private int totalPage;
        private int count;
        private int totalCount;
        private List rows;

        public Builder() {
            this(0, 0, 0, 0, Collections.emptyList());
        }

        public Builder(List rows) {
            this(0, 0, 0, 0, rows);
        }

        public Builder(int page, int totalPage, int count, int totalCount, List rows) {
            this.page = page <= 0 ? 0 : page;
            this.totalPage = totalPage <= 0 ? 0 : totalPage;
            this.count = count <= 0 && rows == null || rows.size() == 0
                    ? 0 : count <= 0 && (rows != null && rows.size() > 0) ? rows.size() : count;
            this.totalCount = totalCount <= 0 ? 0 : totalCount;
            this.rows = rows == null ? Collections.emptyList() : rows;
        }

        public <T> Builder(T t, Class<T> type) {
            this(0, 0, 0, 0, t, type);
        }

        public <T> Builder(int page, int totalPage, int count, int totalCount, T t, Class<T> type) {
            this.page = page <= 0 ? 0 : page;
            this.totalPage = totalPage <= 0 ? 0 : totalPage;
            this.count = count <= 0 ? 0 : count;
            this.totalCount = totalCount <= 0 ? 0 : totalCount;

            if (t == null) {
                this.count = 0;
            } else{
                if (t instanceof List) {
                    this.rows = (List) t;
                } else {
                    List<T> rows = new ArrayList<>();
                    rows.add(t);
                    this.rows = rows;
                }
                this.count = count == 0 ? this.rows.size() : count;
            }
        }

        public Builder setPage(int page) {
            this.page = page;
            return this;
        }

        public Builder setTotalPage(int totalPage) {
            this.totalPage = totalPage;
            return this;
        }

        public Builder setTotalCount(int totalCount) {
            this.totalCount = totalCount;
            return this;
        }

        public Builder setCount(int count) {
            this.count = count;
            return this;
        }

        public Builder setRows(List rows) {
            this.rows = rows;
            this.count = this.count == 0 ? this.rows.size() : this.count;
            return this;
        }

        public BizData build() {
            BizData data = new BizData();
            data.page = this.page;
            data.totalPage = this.totalPage;
            data.count = this.count;
            data.totalCount = this.totalCount;
            data.rows = this.rows;
            return data;
        }

    }

}
