package com.emato.eoss.msg.resp;

import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

/**
 * 回执消息结构
 *
 * @author Scott Chen
 * @date 2017/4/20
 */
public class Result implements Serializable {

    private static final long serialVersionUID = 6695762416670978135L;

    /**
     * 消息码
     */
    private String code;

    /**
     * 消息
     */
    private String message;

    /**
     * 回执业务数据
     */
    private BizData data;

    private Result() {
    }

    public String getCode() {
        return code;
    }

    public String getMessage() {
        return message;
    }

    public BizData getData() {
        return data;
    }

    public static boolean isSuccess(Result message) {
        return message.getCode() == null ? false : message.getCode().equals(ResponseStatus.SUCCESS.code) ? true : false;
    }

    public static Builder builder() {
        return new Builder();
    }
    public static Builder builder(String code, String message) {
        return new Builder(code, message);
    }
    public static Builder builder(String code, String message, BizData bizData) {
        return new Builder(code, message, bizData);
    }
    public static Builder builder(String code, String message, List list) {
        return new Builder(code, message, list);
    }
    public static <T> Builder builder(T t,  Class<T> type) {
        return new Builder(ResponseStatus.SUCCESS.code, ResponseStatus.SUCCESS.msg, t, type);
    }
    public static <T> Builder builder(String message, T t,  Class<T> type) {
        return new Builder(ResponseStatus.SUCCESS.code, message, t, type);
    }
    public static <T> Builder builder(String code, String message, T t,  Class<T> type) {
        return new Builder(code, message, t, type);
    }


    //---------- 直接返回结果 ----------
    //成功
    public static Result success() {
        return new Builder().build();
    }
    public static Result success(String code, String message) {
        return new Builder(code, message).build();
    }
    public static Result success(String message) {
        return new Builder(ResponseStatus.SUCCESS.code, message).build();
    }

    public static Result success(Map map) {
        return new Builder(ResponseStatus.SUCCESS.code, ResponseStatus.SUCCESS.msg, map).build();
    }
    public static Result success(String message, Map map) {
        return new Builder(ResponseStatus.SUCCESS.code, message, map).build();
    }
    public static Result success(List rows) {
        return new Builder(ResponseStatus.SUCCESS.code, ResponseStatus.SUCCESS.msg, rows).build();
    }
    public static Result success(String message, List rows) {
        return new Builder(ResponseStatus.SUCCESS.code, message, rows).build();
    }
    public static Result success(String code, String message, List rows) {
        return new Builder(code, message, rows).build();
    }
    public static <T> Result success(T t, Class<T> type) {
        return new Builder(ResponseStatus.SUCCESS.code, ResponseStatus.SUCCESS.msg, t, type).build();
    }
    public static <T> Result success(String message, T t, Class<T> type) {
        return new Builder(ResponseStatus.SUCCESS.code, message, t, type).build();
    }
    public static <T> Result success(String code, String message, T t, Class<T> type) {
        return new Builder(code, message, t, type).build();
    }

    //错误
    public static Result error() {
        return new Builder(ResponseStatus.ERROR.code, ResponseStatus.ERROR.msg).build();
    }
    public static Result error(String code, String message) {
        return new Builder(code, message).build();
    }
    public static Result error(String message) {
        return new Builder(ResponseStatus.ERROR.code, message).build();
    }
    public static Result error(Map map) {
        return new Builder(ResponseStatus.ERROR.code, ResponseStatus.ERROR.msg, map).build();
    }
    public static Result error(String message, Map map) {
        return new Builder(ResponseStatus.ERROR.code, message, map).build();
    }
    public static Result error(List rows) {
        return new Builder(ResponseStatus.ERROR.code, ResponseStatus.ERROR.msg, rows).build();
    }
    public static Result error(String message, List rows) {
        return new Builder(ResponseStatus.ERROR.code, message, rows).build();
    }
    public static Result error(String code, String message, List rows) {
        return new Builder(code, message, rows).build();
    }

    //失败
    public static Result failed() {
        return new Builder(ResponseStatus.FAILURE.code, ResponseStatus.FAILURE.msg).build();
    }
    public static Result failed(String code, String message) {
        return new Builder(code, message).build();
    }
    public static Result failed(String message) {
        return new Builder(ResponseStatus.FAILURE.code, message).build();
    }

    public static Result failed(Map map) {
        return new Builder(ResponseStatus.ERROR.code, ResponseStatus.ERROR.msg, map).build();
    }
    public static Result failed(String message, Map map) {
        return new Builder(ResponseStatus.ERROR.code, message, map).build();
    }
    public static Result failed(List rows) {
        return new Builder(ResponseStatus.FAILURE.code, ResponseStatus.FAILURE.msg, rows).build();
    }
    public static Result failed(String message, List rows) {
        return new Builder(ResponseStatus.ERROR.code, message, rows).build();
    }
    public static Result failed(String code, String message, List rows) {
        return new Builder(code, message, rows).build();
    }

    /**
     * 消息构建器类
     */
    public static class Builder {
        private String code;
        private String message;
        private BizData data;

        public Builder() {
            this.code = ResponseStatus.SUCCESS.code;
            this.message = ResponseStatus.SUCCESS.msg;
            this.data = BizData.builder().build();
        }

        public Builder(String code, String message) {
            this.code = StringUtils.isBlank(code) ? ResponseStatus.SUCCESS.code : code;
            this.message = StringUtils.isBlank(message) ? ResponseStatus.SUCCESS.msg : message;
            this.data = BizData.builder().build();
        }

        public Builder(String code, String message, BizData bizData) {
            this.code = StringUtils.isBlank(code) ? ResponseStatus.SUCCESS.code : code;
            this.message = StringUtils.isBlank(message) ? ResponseStatus.SUCCESS.msg : message;
            this.data = bizData;
        }

        public Builder(String code, String message, Map map) {
            this.code = StringUtils.isBlank(code) ? ResponseStatus.SUCCESS.code : code;
            this.message = StringUtils.isBlank(message) ? ResponseStatus.SUCCESS.msg : message;
            this.data = BizData.builder(map, Map.class).build();
        }

        public Builder(String code, String message, List rows) {
            this.code = StringUtils.isBlank(code) ? ResponseStatus.SUCCESS.code : code;
            this.message = StringUtils.isBlank(message) ? ResponseStatus.SUCCESS.msg : message;
            this.data = BizData.builder(rows).build();
        }

        public <T> Builder(T t, Class<T> type) {
            this.code = ResponseStatus.SUCCESS.code;
            this.message = ResponseStatus.SUCCESS.msg;
            this.data = BizData.builder(t, type).build();
        }

        public <T> Builder(String message, T t, Class<T> type) {
            this.code = ResponseStatus.SUCCESS.code;
            this.message = StringUtils.isBlank(message) ? ResponseStatus.SUCCESS.msg : message;
            this.data = BizData.builder(t, type).build();
        }

        public <T> Builder(String code, String message, T t, Class<T> type) {
            this.code = StringUtils.isBlank(code) ? ResponseStatus.SUCCESS.code : code;
            this.message = StringUtils.isBlank(message) ? ResponseStatus.SUCCESS.msg : message;
            this.data = BizData.builder(t, type).build();
        }

        public Builder setCode(String code) {
            this.code = code;
            return this;
        }

        public Builder setMessage(String message) {
            this.message = message;
            return this;
        }

        public Builder setData(BizData data) {
            this.data = data;
            return this;
        }

        public Result build() {
            Result message = new Result();
            message.code = this.code;
            message.message = this.message;
            message.data = this.data;
            return message;
        }

    }
}

