package com.emato.eoss.util.sign;

import org.apache.commons.codec.digest.DigestUtils;

import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.Base64;

/** 
 * MD5 digest
 *
 * @author Scott Chen
 * @date 2017/2/27
 */
public class Md5 {

    /**
     * 签名字符串
     * @param text
     * @return
     */
    public static byte[] signMd5(String text) {
        return signMd5(text, null, null);
    }

    /**
     * 签名字符串
     * @param text
     * @param key
     * @param charset
     * @return return byte
     */
    public static byte[] signMd5(String text, String key, Charset charset) {
        charset = charset == null ? StandardCharsets.UTF_8 : charset;
        text = text + (key == null ? "" : key);
        return DigestUtils.md5(text.getBytes(charset));
    }

    /**
     * 签名字符串
     * @param text
     * @return
     */
    public static String signMd5Base64(String text) {
        return signMd5Base64(text, null, null);
    }

    /**
     * 签名字符串
     * @param text
     * @param key
     * @param charset
     * @return return string of Base64
     */
    public static String signMd5Base64(String text, String key, Charset charset) {
        byte[] code = signMd5(text, key, charset);
        return Base64.getEncoder().encodeToString(code);
    }

    /**
     * 签名字符串
     * @param text
     * @return
     */
    public static String signMd5Hex(String text) {
        return signMd5Hex(text, null, null);
    }

    /**
     * 签名字符串
     * @param text 需要签名的字符串
     * @param key 密钥
     * @param charset 编码格式
     * @return 签名结果 MD5 digest as a 32 character hex string
     */
    public static String signMd5Hex(String text, String key, Charset charset) {
        charset = charset == null ? StandardCharsets.UTF_8 : charset;
        text = text + (key == null ? "" : key);
        return DigestUtils.md5Hex(text.getBytes(charset));
    }

    /**
     * 签名字符串
     * @param text
     * @return return string of Base64
     */
    public static String singMd5HexBase64(String text) {
        return singMd5HexBase64(text, null, null);
    }

    /**
     * 签名字符串
     * @param text
     * @param key
     * @param charset
     * @return return string of Base64
     */
    public static String singMd5HexBase64(String text, String key, Charset charset) {
        String code = signMd5Hex(text, key, charset);
        return Base64.getEncoder().encodeToString(code.getBytes());
    }

    /**
     * 验证签名
     * @param text
     * @param sign
     * @return
     */
    public static boolean verifyMd5(String text, String sign) {
        return verifyMd5(text, sign, null, null);
    }

    /**
     * 验证签名
     * @param text
     * @param sign
     * @param key
     * @param charset
     * @return
     */
    public static boolean verifyMd5(String text, String sign, String key, Charset charset) {
        charset = charset == null ? StandardCharsets.UTF_8 : charset;
        text = text + (key == null ? "" : key);
        byte[] mySign = DigestUtils.md5(text.getBytes(charset));
        if(sign.equals(new String(mySign, StandardCharsets.UTF_8))) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * 验证签名
     * @param text
     * @param base64Sign
     * @return
     */
    public static boolean verifyMd5Base64(String text, String base64Sign) {
        return verifyMd5Base64(text, base64Sign, null, null);
    }

    /**
     * 验证签名
     * @param text
     * @param base64Sign
     * @param key
     * @param charset
     * @return
     */
    public static boolean verifyMd5Base64(String text, String base64Sign, String key, Charset charset) {
        charset = charset == null ? StandardCharsets.UTF_8 : charset;
        text = text + (key == null ? "" : key);
        byte[] mySign = DigestUtils.md5(text.getBytes(charset));
        String base64MySign = Base64.getEncoder().encodeToString(mySign);
        if(base64Sign.equals(base64MySign)) {
            return true;
        }
        else {
            return false;
        }
    }

    /**
     * 验证签名
     * @param text
     * @param sign
     * @return
     */
    public static boolean verifyMd5Hex(String text, String sign) {
        return verifyMd5Hex(text, sign, null, null);
    }

    /**
     * 验证签名
     * @param text 需要签名的字符串
     * @param sign 签名结果 MD5 digest as a 32 character hex string
     * @param key 密钥
     * @param charset 编码格式
     * @return
     */
    public static boolean verifyMd5Hex(String text, String sign, String key, Charset charset) {
        charset = charset == null ? StandardCharsets.UTF_8 : charset;
        text = text + (key == null ? "" : key);
    	String mySign = DigestUtils.md5Hex(text.getBytes(charset));
    	if(mySign.equals(sign)) {
    		return true;
    	}
    	else {
    		return false;
    	}
    }

    /**
     * 验证签名
     * @param text
     * @param base64Sign
     * @return
     */
    public static boolean verifyMd5HexBase64(String text, String base64Sign) {
        return verifyMd5HexBase64(text, base64Sign, null, null);
    }

    /**
     * 验证签名
     * @param text
     * @param base64Sign
     * @param key
     * @param charset
     * @return
     */
    public static boolean verifyMd5HexBase64(String text, String base64Sign, String key, Charset charset) {
        charset = charset == null ? StandardCharsets.UTF_8 : charset;
        text = text + (key == null ? "" : key);
        String mySign = DigestUtils.md5Hex(text.getBytes(charset));
        String base64MySign = Base64.getEncoder().encodeToString(mySign.getBytes(charset));
        if(base64MySign.equals(base64Sign)) {
            return true;
        }
        else {
            return false;
        }
    }

}